package com.ruoyi.asset.mapper;

import java.util.Collection;
import java.util.List;
import com.ruoyi.asset.domain.TbAssetInformation;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

/**
 * 资产信息Mapper接口
 * 
 * @author 原动力
 * @date 2023-05-15
 */
public interface TbAssetInformationMapper 
{
    /**
     * 查询资产信息
     * 
     * @param id 资产信息主键
     * @return 资产信息
     */
    public TbAssetInformation selectTbAssetInformationById(Long id);

    List<TbAssetInformation> selectTbAssetInformationByIds(List<Long> ids);

    /**
     * 查询资产信息
     *
     * @param assetNumber 资产编号
     * @return 资产信息
     */
    TbAssetInformation selectTbAssetInformationByAssetNumber(String assetNumber);

    /**
     * 查询资产信息
     *
     * @param epc epc
     * @return 资产信息
     */
    TbAssetInformation selectTbAssetInformationByEpc(String epc);

    /**
     * 查询资产信息
     * @param epcList epc集合
     * @return
     */
    public List<TbAssetInformation> selectTbAssetInformationByEpcList(@Param("epcList") Collection<String> epcList);

    /**
     * 查询资产信息列表
     * 
     * @param tbAssetInformation 资产信息
     * @return 资产信息集合
     */
    public List<TbAssetInformation> selectTbAssetInformationList( TbAssetInformation tbAssetInformation);

    /**
     * 查询把部门ID替换为名称的资产列表
     * @param tbAssetInformation
     * @return
     */
    public List<TbAssetInformation> selectInfoList(TbAssetInformation tbAssetInformation);

    /**
     * 根据位置编码获取资产列表
     * @param locationNumbers
     * @return
     */
    public List<TbAssetInformation> selectTbAssetInformationByLocationNumbers(@Param("locationNumbers") List<String> locationNumbers);

    /**
     * 新增资产信息
     * 
     * @param tbAssetInformation 资产信息
     * @return 结果
     */
    public int insertTbAssetInformation(TbAssetInformation tbAssetInformation);

    /**
     * 修改资产信息
     * 
     * @param tbAssetInformation 资产信息
     * @return 结果
     */
    public int updateTbAssetInformation(TbAssetInformation tbAssetInformation);

    /**
     * 删除资产信息
     * 
     * @param id 资产信息主键
     * @return 结果
     */
    public int deleteTbAssetInformationById(Long id);

    /**
     * 批量删除资产信息
     * 
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteTbAssetInformationByIds(Long[] ids);

    public boolean updateProperty(TbAssetInformation information);

    List<String> selectNumberByLocations(@Param("locations") List<String> locations);

    List<TbAssetInformation> selectAssetInformationByLocations(@Param("locations") Collection<String> locations);
}
