package com.ruoyi.inventory.mapper;

import java.util.List;
import com.ruoyi.inventory.domain.TbInventoryDetail;
import org.apache.ibatis.annotations.Param;

/**
 * 资产盘点明细Mapper接口
 * 
 * @author ydl
 * @date 2023-06-01
 */
public interface TbInventoryDetailMapper 
{
    /**
     * 查询资产盘点明细
     * 
     * @param id 资产盘点明细主键
     * @return 资产盘点明细
     */
    public TbInventoryDetail selectTbInventoryDetailById(Long id);

    /**
     * 查询资产盘点明细列表
     * 
     * @param tbInventoryDetail 资产盘点明细
     * @return 资产盘点明细集合
     */
    public List<TbInventoryDetail> selectTbInventoryDetailList(TbInventoryDetail tbInventoryDetail);

    /**
     * 查询资产盘点明细列表
     * @param orderNumber 盘点编码
     * @return 资产盘点明细集合
     */
    public List<TbInventoryDetail> selectTbInventoryDetailByOrderNumber(@Param("orderNumber") String orderNumber);

    /**
     * 统计未盘点资产
     * @param orderNumber 盘点编码
     * @return
     */
    public int countUncounted(@Param("orderNumber") String orderNumber);

    /**
     * 统计盘亏
     * @param orderNumber 盘点编码
     * @return
     */
    public int countInventoryShortages(@Param("orderNumber") String orderNumber);

    /**
     * 统计盘盈
     * @param orderNumber 盘点编码
     * @return
     */
    public int countInventoryProfit(@Param("orderNumber") String orderNumber);

    /**
     * 新增资产盘点明细
     * 
     * @param tbInventoryDetail 资产盘点明细
     * @return 结果
     */
    public int insertTbInventoryDetail(TbInventoryDetail tbInventoryDetail);

    /**
     * 修改资产盘点明细
     * 
     * @param tbInventoryDetail 资产盘点明细
     * @return 结果
     */
    public int updateTbInventoryDetail(TbInventoryDetail tbInventoryDetail);

    /**
     * 删除资产盘点明细
     * 
     * @param id 资产盘点明细主键
     * @return 结果
     */
    public int deleteTbInventoryDetailById(Long id);

    public int deleteTbInventoryDetailByOrderNumber(@Param("orderNumber") String orderNumber);

    /**
     * 删除资产盘点明细
     *
     * @param orderNumbers 资产编码
     * @return 结果
     */
    public int deleteTbInventoryDetailByOrderNumbers(@Param("orderNumbers") List<String> orderNumbers);

    /**
     * 批量删除资产盘点明细
     * 
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteTbInventoryDetailByIds(Long[] ids);
}
